using System;
using System.Security;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;
using System.Runtime.ConstrainedExecution;
using System.Security.Principal;

namespace PowerShellRemoting
{
    internal sealed class SafeTokenHandle : SafeHandleZeroOrMinusOneIsInvalid
    {
        private SafeTokenHandle() : base(true) { }

        // 0 is an Invalid Handle
        internal SafeTokenHandle(IntPtr handle)
            : base(true)
        {
            SetHandle(handle);
        }

        internal static SafeTokenHandle InvalidHandle
        {
            get { return new SafeTokenHandle(IntPtr.Zero); }
        }

        override protected bool ReleaseHandle()
        {
            return NativeMethods.CloseHandle(handle);
        }

        public WindowsIdentity GetWindowsIdenty()
        {
            if (!this.IsInvalid)
            {
                WindowsIdentity Id = null;
                try
                {
                    Id = new WindowsIdentity(this.handle);
                }
                catch (Exception ex)
                {
                    throw new ArgumentException("Invalid Token", ex);
                }
                return Id;
            }
            else { return null; }
        }

    }
}
