using System;
using System.Collections.Generic;
using System.Text;
using System.Management.Automation;

namespace PowerShellRemoting
{
    internal class InputLoop
    {
        internal InputLoop(ServerPSHost parent, bool isNested)
        {
            this.parent = parent;
            this.isNested = isNested;
            this.innerExecutor = new Executor(parent, isNested, false);
            this.promptExecutor = new Executor(parent, isNested, true);
        }

        private bool isNested;
        private bool shouldExit;
        private ServerPSHost parent;

        private Executor innerExecutor;
        private Executor promptExecutor;

        internal bool ShouldExit
        {
            get { return this.shouldExit; }
            set { this.shouldExit = value; }
        }
        
        private string EvaluatePrompt()
        {
            Exception exception1 = null;
            string text1 = this.promptExecutor.ExecuteCommandAndGetResultAsString("prompt", out exception1);
            if (exception1 != null)
            {
                this.parent.ReportException(exception1);
            }
            if (string.IsNullOrEmpty(text1))
            {
                text1 = "PS> ";
            }
            return text1;
        }
        
        internal void Run()
        {
            ServerPSHostUserInterface ServerUI = (ServerPSHostUserInterface) this.parent.UI;
            bool MultipleLineMode = false;
            StringBuilder builder1 = null;
            try
            {
                while (!this.parent.ShouldExit && !this.shouldExit)
                {
                    this.parent.IsRunningPromptLoop = true;
                    string text1 = null;
                    if (MultipleLineMode)
                    {
                        text1 = ">> ";
                    }
                    else
                    {
                        ServerUI.WriteLine();
                        text1 = this.EvaluatePrompt();
                    }
                    ServerUI.Write(text1);
                    string text2 = ServerUI.ReadLine(); //ReadLineWithTabCompletion(this.exec);
                    if (text2 == null)
                    {
                        ServerUI.WriteLine();
                        MultipleLineMode = false;
                        continue;
                    }
                    if (text2.Trim().Length == 0)
                    {
                        if (!MultipleLineMode)
                        {
                            ServerUI.WriteLine();
                            continue;
                        }
                        text2 = builder1.ToString();
                        MultipleLineMode = false;
                    }
                    else if (MultipleLineMode)
                    {
                        builder1.Append("\n");
                        builder1.Append(text2);
                        continue;
                    }
                    ServerUI.WriteLine();
                    Exception exception1 = null;
                    if (this.parent.IsRunningAsync)
                    {
                        //this.innerExecutor.ExecuteCommandAsync(text2, out exception1, Executor.ExecutionOptions.AddToHistory | Executor.ExecutionOptions.AddOutputter);
                    }
                    else
                    {
                        this.innerExecutor.ExecuteCommand(text2, out exception1, Executor.ExecutionOptions.AddToHistory | Executor.ExecutionOptions.AddOutputter);
                    }
                    if (exception1 is IncompleteParseException)
                    {
                        if (!MultipleLineMode)
                        {
                            MultipleLineMode = true;
                            builder1 = new StringBuilder(text2);
                        }
                        else
                        {
                            builder1.Append(text2);
                        }

                    }
                    else if (exception1 != null)
                    {
                        this.parent.ReportException(exception1);
                    }
                }
            }
            catch (Exception ex)
            {
                ServerPSHost.CheckForSevereException(ex);
                this.parent.ReportException(ex);
            }
            finally
            {
                this.parent.IsRunningPromptLoop = false;
            }
        }


    }
}
