using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Security.AccessControl;
using System.Security.Principal;
using System.Net;

namespace PowerShellRemoting
{
    public class NetworkACE
    {
        private Collection<IPAddress> iPs;
        public Collection<IPAddress> IPs
        {
            get { return this.iPs; }
        }

        private Collection<Subnet> subnets;
        public Collection<Subnet> Subnets
        {
            get { return this.subnets; }
        }

        private NTAccount user;
        public NTAccount User
        {
            get { return this.user; }
        }

        private bool allow;
        public bool Allow
        {
            get { return this.allow; }
        }

        private SecurityIdentifier sid;
        public SecurityIdentifier Sid
        {
            get{ return this.sid; }
        }

        public NetworkACE(NTAccount User, Collection<IPAddress> IPs, Collection<Subnet> Subs, bool Allow)
        {
            if (User == null)
                throw new ArgumentNullException("User");
            if (IPs == null)
                throw new ArgumentNullException("IPs");
            if (Subs == null)
                throw new ArgumentNullException("Subs");
            if (IPs.Count == 0 && Subs.Count == 0)
                throw new ArgumentOutOfRangeException("IPs|Subs");
            this.iPs = IPs;
            this.subnets = Subs;
            this.allow = Allow;
            this.user = User;
            this.sid = (SecurityIdentifier)this.user.Translate(typeof(SecurityIdentifier));
        }

        public override string ToString()
        {
            StringBuilder buf = new StringBuilder();
            if (this.allow)
                buf.Append("Allow ");
            else
                buf.Append("Deny ");
            buf.Append(this.User.Value);
            buf.Append(" from ");
            buf.Append("{ ");
            if (this.iPs.Count > 0)
            {
                foreach (IPAddress ip in this.iPs)
                {
                    buf.Append(ip.ToString());
                    buf.Append("; ");
                }
                if (this.subnets.Count == 0)
                    buf.Remove(buf.Length - 2, 2);
            }
            if (this.subnets.Count > 0)
            {
                foreach (Subnet sub in this.subnets)
                {
                    buf.Append(sub.ToString());
                    buf.Append("; ");
                }
                buf.Remove(buf.Length - 2, 2);
            }
            buf.Append(" }");
            return buf.ToString();

        }
    }
}
