using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Security.Principal;
using System.Net;

namespace PowerShellRemoting
{
    public class NetworkACL
    {
        private Collection<NetworkACE> innerNetworkACEList;

        public NetworkACL()
        {
            this.innerNetworkACEList = new Collection<NetworkACE>() ;
        }

        public void Add(NetworkACE ace)
        {
            if (ace == null)
                throw new ArgumentNullException("ace");
            if (!this.innerNetworkACEList.Contains(ace))
                this.innerNetworkACEList.Add(ace);
        }

        public bool AccessAllowed(WindowsIdentity Id, IPAddress Ip)
        {
            if (Id == null)
                throw new ArgumentNullException("Id");
            if (Ip == null)
                throw new ArgumentNullException("Ip");

            int DenyAsUser = 0;
            int DenyAsGroup = 0;
            int AllowAsUsr = 0;
            int AllowAsGroup = 0;

            foreach (NetworkACE ace in this.innerNetworkACEList)
            {
                if (Id.Owner.Equals(ace.Sid))
                {
                    if (this.MatchIP(Ip, ace))
                    {
                        if (ace.Allow)
                            AllowAsUsr++;
                        else
                            DenyAsUser++;
                        continue;
                    }
                    continue;
                }
                else if (this.MatchGroup(Id, ace))
                {
                    if (this.MatchIP(Ip, ace))
                    {
                        if (ace.Allow)
                            AllowAsGroup++;
                        else
                            DenyAsGroup++;
                    }
                }                
            }
            if (DenyAsUser > 0)
                return false;
            if (AllowAsUsr > 0)
                return true;
            if (DenyAsGroup > 0)
                return false;
            if (AllowAsGroup > 0)
                return true;
            return false;
        }

        private bool MatchGroup(WindowsIdentity Id, NetworkACE ace)
        {
            SecurityIdentifier CurrentSid, UserGroupSid;
            CurrentSid = ace.Sid;
            foreach (IdentityReference CurrentUserGroup in Id.Groups)
            {
                UserGroupSid = (SecurityIdentifier)CurrentUserGroup;
                if (CurrentSid.Equals(UserGroupSid))
                    return true;
            }
            return false; 
        }

        private bool MatchIP(IPAddress IP, NetworkACE ace)
        {
            if (ace.IPs.Count > 0 && ace.IPs.Contains(IPAddress.Any))
                return true;
            if (ace.IPs.Count > 0 && ace.IPs.Contains(IP))
                return true;
            if (IP.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
                return false;
            uint SubnetIP;
            if (ace.Subnets.Count > 0)
            {
                foreach (Subnet sub in ace.Subnets)
                {
                    if (!Subnet.TryParseIP(IP.ToString(), out SubnetIP))
                        continue;
                    SubnetIP = SubnetIP & sub.Mask;
                    if (SubnetIP == sub.Address)
                        return true; 
                }
            }
            return false;
        }

        public override string ToString()
        {
            if (this.innerNetworkACEList.Count == 0)
                return string.Empty;
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < this.innerNetworkACEList.Count; i++)
            {
                buf.Append("(" + i.ToString() + ") ");
                buf.Append(this.innerNetworkACEList[i].ToString());
                buf.Append("\n\r");
            }
            return buf.ToString();
        }

    }
}
