using System;
using System.Collections;
using System.Text;
using System.Threading;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.ServiceProcess;
using System.Net;
using System.Net.Sockets;
using System.Security.Principal;
using System.Xml;
using System.IO;
using System.Security.Cryptography.X509Certificates;



namespace PowerShellRemoting
{
    public class PowerShellRemotingService : ServiceBase
    {
        private Thread ListernerThread = null;
        private Thread ReclaimThread = null;
        private ArrayList PSHosts = null;
        private int MaximumClientNum = 0;
        private int MaximumClientPerIP = 0;
        private int Port = 0;
        private TcpListener MonadListener = null;
        private bool Running = false;
        private NetworkACL ACL = null;
        private bool DebugHost = false;
        private X509Certificate X509 = null;

        public const int EventStartService = 1001;
        public const int EventStopService = 1002;
        public const int EventACLCanNotFind = 1003;
        public const int EventACLParseError = 1004;
        public const int EventACLLoadDefault = 1005;
        public const int EventACLMaxClient = 1006;
        public const int EventACLMaxClientPerIP = 1007;
        public const int EventACLDenied = 1008;
        public const int EventErrorNotSupported = 1009;
        public const int EventErrorUnknown = 1009;


        public PowerShellRemotingService()
        {
            this.ServiceName = "PowerShellRemoting";
            this.CanShutdown = true;
            this.CanStop = true;
            this.CanPauseAndContinue = false;
            this.AutoLog = false;

        }

        internal bool Debug
        {
            get { return this.DebugHost; }
        }

        internal string UserACL
        {
            get { return this.ACL.ToString(); }
        }

        internal X509Certificate X509Cer
        {
            get { return this.X509; }
        }
  
        static void Main()
        {
            PowerShellRemotingService Server = new PowerShellRemotingService();
            if (Server != null)
                ServiceBase.Run(Server);
        }

        protected override void OnStart(string[] args)
        {
            try
            {
                this.Running = true;
                SetParameters();
                this.LoadAccessControl();
                this.MonadListener = new TcpListener(new IPEndPoint(IPAddress.Any, this.Port));
                PSHosts = new ArrayList();
                if ((ReclaimThread == null) ||
                    ((ReclaimThread.ThreadState &
                     (System.Threading.ThreadState.Unstarted | System.Threading.ThreadState.Stopped)) != 0))
                {
                    ReclaimThread = new Thread(new ThreadStart(Reclaim));
                    ReclaimThread.Start();
                }
                if ((ListernerThread == null) ||
                    ((ListernerThread.ThreadState &
                     (System.Threading.ThreadState.Unstarted | System.Threading.ThreadState.Stopped)) != 0))
                {
                    ListernerThread = new Thread(new ThreadStart(ServiceWorkerMethod));
                    ListernerThread.Start();
                }
                this.EventLog.WriteEntry("Service started!",EventLogEntryType.Information,PowerShellRemotingService.EventStartService);
            }
            catch (Exception ex)
            {
                this.EventLog.WriteEntry(string.Format("{0}\n\r{1}",ex.Message,ex.StackTrace),EventLogEntryType.Error,PowerShellRemotingService.EventErrorUnknown);
                this.OnStop();
            }
        }

        protected override void OnStop()
        {
            
            this.Running = false;
            Thread.Sleep(1000);
            if (this.MonadListener != null)
            {
                this.MonadListener.Stop();
                this.MonadListener = null;
            }
            if ((ListernerThread != null) && (ListernerThread.IsAlive))
                ListernerThread.Abort();
            if ((ReclaimThread != null) && (ReclaimThread.IsAlive))
                ReclaimThread.Join();
            if (this.PSHosts != null)
            {
                lock (this.PSHosts.SyncRoot)
                {
                    for (int i = 0; i < this.PSHosts.Count; i++)
                    {
                        if (this.PSHosts[i] != null)
                        {
                            ServerPSHost Host = (ServerPSHost)this.PSHosts[i];
                            Host.SetShouldExit(ServerPSHost.ServiceShutDown);
                            Host.Dispose();
                            Host = null;
                        }
                    }
                }
                this.PSHosts = null;
            }
            if (this.ACL != null)
                this.ACL = null;
            this.EventLog.WriteEntry("Service stopped",EventLogEntryType.Information,PowerShellRemotingService.EventStopService);
            this.ExitCode = 0;
        }

        public void ServiceWorkerMethod()
        {
            ServerPSHost Host = null;
            TcpClient IncomeClient = null;
            if (this.MonadListener == null)
                return;
            try
            {
                this.MonadListener.Start();
                this.EventLog.WriteEntry("Listening at Port: " + this.Port.ToString()
                    + "\nMaximumClient: " + this.MaximumClientNum.ToString()
                    + "\nMaximumClientPerIP: " + this.MaximumClientPerIP.ToString()
                    + "\nDebugHost: " + this.DebugHost.ToString()
                    + "\nSSL: " + (this.X509 == null ? "Not Supported" : "Supported"),
                EventLogEntryType.Information, PowerShellRemotingService.EventStartService);
                string Message;
                while (this.Running)
                {
                    if (!this.MonadListener.Pending())
                    {
                        Thread.Sleep(1000);
                        continue;
                    }
                    IncomeClient = MonadListener.AcceptTcpClient();
                    if (IncomeClient == null)
                        continue;
                    if (this.PSHosts.Count >= this.MaximumClientNum)
                    {
                        this.EventLog.WriteEntry("Reach Maximum Client Capacity!",EventLogEntryType.Warning,PowerShellRemotingService.EventACLMaxClient);
                        Message = "~MaxClient";
                        this.HandShake(IncomeClient, ref Message);
                        IncomeClient.Close();
                        continue;
                    }
                    if (this.ReachMaxClientPerIP(IncomeClient))
                    {
                        this.EventLog.WriteEntry("Reach Maximum Client Per IP Address(" + ((IPEndPoint)(IncomeClient.Client.RemoteEndPoint)).Address.ToString() + ")!",EventLogEntryType.Warning,PowerShellRemotingService.EventACLMaxClientPerIP);
                        Message = "~MaxClientPerIP";
                        this.HandShake(IncomeClient, ref Message);
                        IncomeClient.Close();
                        continue;
                    }
                    Message = "~Welcome";
                    this.HandShake(IncomeClient, ref Message);
                    if (Message == "~SSLChannel")
                    {
                        if (this.X509 == null)
                        {
                            this.EventLog.WriteEntry("SSLChannel is not supported by server!", EventLogEntryType.Warning, PowerShellRemotingService.EventErrorNotSupported);
                            Message = "~NoSupport";
                            this.HandShake(IncomeClient, ref Message);
                            IncomeClient.Close();
                            continue;
                        }
                        else
                        {
                            Message = "~Welcome";
                            this.HandShake(IncomeClient, ref Message);
                        }
                    }
                    Host = new ServerPSHost(IncomeClient, this, Message);
                    Host.InitializeChannel();
                    if (Host.ShouldExit)
                    {
                        Host.Dispose();
                        Host = null;
                        continue;
                    }
                    if (!this.ACL.AccessAllowed(Host.RemoteId, ((IPEndPoint)IncomeClient.Client.RemoteEndPoint).Address))
                    {
                        this.EventLog.WriteEntry("User(" + Host.RemoteId.Name + ") login from " + ((IPEndPoint)(IncomeClient.Client.RemoteEndPoint)).Address.ToString() + " is denied!", EventLogEntryType.Warning, PowerShellRemotingService.EventACLDenied);
                        Host.UserBanned();
                        Host.Dispose();
                        Host = null;
                        continue;
                    }
                    lock (this.PSHosts.SyncRoot)
                    {
                        this.PSHosts.Add(Host);
                    }
                    ThreadPool.QueueUserWorkItem(new WaitCallback(Host.Run), this.Running);
                }
            }
            catch (Exception ex)
            {
                if ((ex is AccessViolationException) || (ex is StackOverflowException))
                {
                    this.Running = false;
                    Environment.FailFast(ex.Message);
                }
                if (!(ex is ThreadAbortException))
                    this.EventLog.WriteEntry(string.Format("{0}\n\r{1}", ex.Message, ex.StackTrace), EventLogEntryType.Error,PowerShellRemotingService.EventErrorUnknown);
            }
            finally
            {
                if (Host != null)
                {
                    Host.SetShouldExit(ServerPSHost.ServiceShutDown);
                    Host.Dispose();
                    Host = null;
                }
                if (IncomeClient != null)
                {
                    IncomeClient.Close();
                    IncomeClient = null;
                }
                if (this.MonadListener != null)
                {
                    this.MonadListener.Stop();
                    this.MonadListener = null;
                }
                this.Running = false;
            }
        }

        private void SetParameters()
        {
            Microsoft.Win32.RegistryKey system, currentControlSet,services,service,config;
            system = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("System");
            currentControlSet = system.OpenSubKey("CurrentControlSet");
            services = currentControlSet.OpenSubKey("Services");
            service = services.OpenSubKey("PowerShellRemoting");
            config = service.OpenSubKey("Parameters");
            this.Port = (int)config.GetValue("Port", 8080);
            this.MaximumClientNum = (int)config.GetValue("MaxClient", 10);
            this.MaximumClientPerIP = (int)config.GetValue("MaxClientPerIP", 2);
            this.DebugHost = ((int)config.GetValue("DebugHost", 0))>0?true:false;
            string Thumbprint = (string)config.GetValue("X509", "0");
            if (this.Port <= 0 || this.Port < IPEndPoint.MinPort || this.Port > IPEndPoint.MaxPort)
                this.Port = 8080;
            if (this.MaximumClientNum <= 0)
                this.MaximumClientNum = 10;
            if (Thumbprint != "0")
            {
                X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
                store.Open(OpenFlags.ReadOnly);
                X509CertificateCollection cert = store.Certificates.Find(X509FindType.FindByThumbprint, Thumbprint, true);
                //"f7 c3 4e 5b d4 54 75 b3 7f 03 31 59 15 43 9e 22 8c 1f b5 e9"
                if (cert.Count > 0)
                    this.X509 = cert[0];
                else
                {
                    this.EventLog.WriteEntry("Can't find X509 Cerificate with Thumbprint ", EventLogEntryType.Warning, PowerShellRemotingService.EventErrorUnknown);
                    this.X509 = null;
                }
            }
            else
            {
                this.EventLog.WriteEntry("No X509 Cerificate", EventLogEntryType.Information, PowerShellRemotingService.EventStartService);
                this.X509 = null;
            }
        }

        private bool ReachMaxClientPerIP(TcpClient client)
        {
            if (client == null)
                throw new ArgumentNullException("client");
            IPAddress RunningIP;
            int j = 0;
            ServerPSHost Host;
            IPAddress IP = ((IPEndPoint)(client.Client.RemoteEndPoint)).Address;
            lock (this.PSHosts.SyncRoot)
            {
                for (int i = 0; i < this.PSHosts.Count; i++)
                {
                    if (this.PSHosts[i] != null)
                    {
                        Host = (ServerPSHost)this.PSHosts[i];
                        RunningIP = ((IPEndPoint)(Host.ServerTcpClient.Client.RemoteEndPoint)).Address;
                        if (IP.Equals(RunningIP))
                        {
                            j++;
                        }
                    }
                }
            }
            if (j >= this.MaximumClientPerIP)
                return true;
            else
                return false;
        }

        private void LoadAccessControl()
        {
            this.ACL = new NetworkACL();
            string Path = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + "\\PowerShell Remoting\\user.xml";
            if (!File.Exists(Path))
            {
                this.EventLog.WriteEntry("Can't find " + Path,EventLogEntryType.Warning,PowerShellRemotingService.EventACLCanNotFind);
                this.LoadDefaultACL();
            }
            else
            {
                try
                {
                    this.LoadACLFromXML(Path);
                }
                catch (Exception ex)
                {
                    this.EventLog.WriteEntry(ex.Message, EventLogEntryType.Error,PowerShellRemotingService.EventACLParseError);
                    this.LoadDefaultACL();
                }
            }
        }

        private void LoadDefaultACL()
        {
            this.EventLog.WriteEntry("Load default access control list.", EventLogEntryType.Information,PowerShellRemotingService.EventACLLoadDefault);
            this.ACL = new NetworkACL();
            Collection<IPAddress> IPs = new Collection<IPAddress>();
            IPs.Add(IPAddress.Loopback);
            Collection<Subnet> Subs = new Collection<Subnet>();
            NetworkACE ace = new NetworkACE(new NTAccount("Administrators"), IPs, Subs, true);
            this.ACL.Add(ace);
        }

        private void LoadACLFromXML(string Path)
        {
            XmlReader reader = null;
            try
            {
                this.ACL = new NetworkACL();
                NetworkACE ace = null;
                Collection<IPAddress> IPs = null;
                Collection<Subnet> Subs = null;
                IPAddress IP = null;
                Subnet sub = null;
                NTAccount User = null;
                bool Access = false;
                XmlTextReader textreader = new XmlTextReader(Path);
                XmlReaderSettings settings = new XmlReaderSettings();
                settings.IgnoreComments = true;
                settings.IgnoreProcessingInstructions = true;
                settings.IgnoreWhitespace = true;
                reader = XmlReader.Create(textreader, settings);
                string tmp;
                reader.ReadStartElement("NetworkACL");
                while (reader.Name == "NetworkACE")
                {
                    ace = null;
                    IPs = new Collection<IPAddress>();
                    Subs = new Collection<Subnet>();
                    reader.ReadStartElement("NetworkACE");
                    reader.ReadStartElement("Account");
                    User = new NTAccount(reader.ReadString());
                    reader.ReadEndElement();
                    reader.ReadStartElement("Access");
                    if (reader.ReadString().ToLower() == "allow")
                        Access = true;
                    else
                        Access = false;
                    reader.ReadEndElement();
                    reader.ReadStartElement("IPRange");
                    while (reader.Name == "IP" || reader.Name == "Subnet")
                    {
                        IP = null;
                        sub = null;
                        if (reader.Name == "IP")
                        {
                            reader.ReadStartElement("IP");
                            tmp = reader.ReadString();
                            if (!IPAddress.TryParse(tmp, out IP))
                                throw new InvalidCastException(tmp);
                            else
                                IPs.Add(IP);
                            reader.ReadEndElement();
                        }
                        else if (reader.Name == "Subnet")
                        {
                            reader.ReadStartElement("Subnet");
                            tmp = reader.ReadString();
                            if (!Subnet.TryParseSubnet(tmp, out sub))
                                throw new InvalidCastException(tmp);
                            else
                                Subs.Add(sub);
                            reader.ReadEndElement();
                        }
                    }
                    if (User != null && (IPs.Count > 0 || Subs.Count > 0))
                    {
                        ace = new NetworkACE(User, IPs, Subs, Access);
                        this.ACL.Add(ace);
                    }
                    else
                        throw new ArgumentException("Invalid NetworkACE");
                    reader.ReadEndElement();//</IPRange>
                    reader.ReadEndElement();//</NetworkACE>
                }
                reader.ReadEndElement();
            }
            finally
            {
                if (reader != null && reader.ReadState != ReadState.Closed)
                    reader.Close();
            }
        }


        private void Reclaim()
        {
            ServerPSHost Host;

            try
            {
                while (this.Running)
                {
                    lock (this.PSHosts.SyncRoot)
                    {
                        for (int i = 0; i < this.PSHosts.Count; i++)
                        {
                            if (this.PSHosts[i] == null)
                            {
                                this.PSHosts.RemoveAt(i);
                            }
                            else
                            {
                                Host = (ServerPSHost)this.PSHosts[i];
                                if (Host.ShouldExit)
                                {
                                    Host.Dispose();
                                    Host = null;
                                    this.PSHosts.RemoveAt(i);
                                }
                            }
                        }
                    }
                    Thread.Sleep(2000);
                }
            }
            catch (Exception ex)
            {
                this.EventLog.WriteEntry(string.Format("{0}\n\r{1}", ex.Message, ex.StackTrace),EventLogEntryType.Error, PowerShellRemotingService.EventErrorUnknown);
            }
        }

        private void HandShake(TcpClient client, ref string Message)
        {
            if (client == null)
                throw new ArgumentNullException("client");
            if (string.IsNullOrEmpty(Message))
                throw new ArgumentNullException("Message");
            NetworkStream stream = null;
            byte[] MessageBytes = new byte[1024];
            int length;
            string IncomeMessage = string.Empty;
            stream = client.GetStream();
            stream.ReadTimeout = 5000;
            stream.WriteTimeout = 5000;
            length = stream.Read(MessageBytes,0,MessageBytes.Length);
            IncomeMessage = Encoding.ASCII.GetString(MessageBytes,0,length);
            MessageBytes = Encoding.ASCII.GetBytes(Message);
            stream.Write(MessageBytes, 0, MessageBytes.Length);
            stream.ReadTimeout = System.Threading.Timeout.Infinite;
            stream.WriteTimeout = System.Threading.Timeout.Infinite;
            Message = IncomeMessage;
        }

    }

}
