using System;
using System.Collections.Generic;
using System.Text;
using System.Management.Automation;
using System.Management.Automation.Host;
using System.Management.Automation.Runspaces;

namespace PowerShellRemoting
{
    internal class ServerPSHostRawUserInterface : PSHostRawUserInterface
    {
        public ServerPSHostRawUserInterface(ServerPSHostUserInterface Parent)
        {
            if (Parent == null)
                throw new ArgumentNullException("Parent");
            this.parent = Parent;
        }

        private ServerPSHostUserInterface parent = null;
        
        public override ConsoleColor BackgroundColor
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetBackgroundColor);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command == SecureChannel.Error)
                    throw new InvalidOperationException("Get BackgroundColor");
                ConsoleColor Color = (ConsoleColor)command;
                return Color; 
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetBackgroundColor);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set BackgroundColor");
                this.parent.Parent.MyChannel.SendCommand((int)value);
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set BackgroundColor");
            }
        }

        public override ConsoleColor ForegroundColor
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetForegroundColor);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command == SecureChannel.Error)
                    throw new InvalidOperationException("Get ForegroundColor");
                ConsoleColor Color = (ConsoleColor)command;
                return Color;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetForegroundColor);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set ForegroundColor");
                this.parent.Parent.MyChannel.SendCommand((int)value);
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set ForegroundColor");
            }
        }

        public override Size BufferSize
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetBufferSize);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get BufferSize");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Size buffer = new Size(Int32.Parse(split[0]),Int32.Parse(split[1]));
                return buffer;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetBufferSize);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set BufferSize");
                this.parent.Parent.MyChannel.WriteText(value.Width.ToString() + "~" + value.Height.ToString());
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set BufferSize");
            }
        }

        public override Coordinates CursorPosition
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetCursorPosition);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get CursorPosition");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Coordinates coursor = new  Coordinates(Int32.Parse(split[0]), Int32.Parse(split[1]));
                return coursor;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetCursorPosition);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set CursorPosition");
                this.parent.Parent.MyChannel.WriteText(value.X.ToString() + "~" + value.Y.ToString());
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set CursorPosition");
            }
        }

        public override int CursorSize
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetCursorSize);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command == SecureChannel.Error)
                    throw new InvalidOperationException("Get CursorSize");
                return command;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetCursorSize);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set CursorSize");
                this.parent.Parent.MyChannel.SendCommand((int)value);
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set CursorSize");
            }
        }

        public override void FlushInputBuffer()
        {
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawFlushInputBuffer);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("FlushInputBuffer");
        }

        public override bool KeyAvailable
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawKeyAvailable);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command == SecureChannel.Error)
                    throw new InvalidOperationException("Get KeyAvailable");
                return  command == 1?true:false;
            }
        }

        public override Size MaxPhysicalWindowSize
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetMaxPhysicalWindowSize);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get MaxPhysicalWindowSize");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Size window = new Size(Int32.Parse(split[0]), Int32.Parse(split[1]));
                return window;
            }
        }

        public override Size MaxWindowSize
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetMaxWindowSize);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get MaxWindowSize");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Size window = new Size(Int32.Parse(split[0]), Int32.Parse(split[1]));
                return window;
            }
        }

        public override Coordinates WindowPosition
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetWindowPosition);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get WindowPosition");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Coordinates window = new Coordinates(Int32.Parse(split[0]), Int32.Parse(split[1]));
                return window;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetWindowPosition);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowPosition");
                this.parent.Parent.MyChannel.WriteText(value.X.ToString() + "~" + value.Y.ToString());
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowPosition");
            }
        }

        public override Size WindowSize
        {
            get
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetWindowSize);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get WindowSize");
                string[] split = data.Split(new Char[] { '~' });
                if (split.Length != 2)
                    throw new ArgumentOutOfRangeException("split");
                Size window = new Size(Int32.Parse(split[0]), Int32.Parse(split[1]));
                return window;
            }
            set
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetWindowSize);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowSize");
                this.parent.Parent.MyChannel.WriteText(value.Width.ToString() + "~" + value.Height.ToString());
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowSize");
            }
        }

        public override string WindowTitle
        {
            get 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetWindowTitle);
                string data = this.parent.Parent.MyChannel.ReadText();
                if (data == SecureChannel.ErrorLine)
                    throw new InvalidOperationException("Get WindowTitle");
                if (data == SecureChannel.EmptyLine)
                    return string.Empty;
                else
                    return data;
            }
            set 
            {
                this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetWindowTitle);
                int command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowTitle");
                if (string.IsNullOrEmpty(value))
                    this.parent.Parent.MyChannel.WriteText(SecureChannel.EmptyLine);
                else
                    this.parent.Parent.MyChannel.WriteText(value);
                command = this.parent.Parent.MyChannel.ReceiveCommand();
                if (command != SecureChannel.OK)
                    throw new InvalidOperationException("Set WindowTitle");
            }
        }


        public override KeyInfo ReadKey(ReadKeyOptions options)
        {
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawReadKey);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("ReadKey");
            this.parent.Parent.MyChannel.SendCommand((int)options);
            this.parent.Parent.MyChannel.UnsetReadTimeout();
            string data = this.parent.Parent.MyChannel.ReadText();
            this.parent.Parent.MyChannel.SetReadTimeout(5000);
            if (data == SecureChannel.ErrorLine)
                throw new InvalidOperationException("ReadKey");
            string[] split = data.Split(new Char[] { '~' });
            if (split.Length != 4)
                throw new ArgumentOutOfRangeException("split");
            KeyInfo Key = new KeyInfo();
            if (split[0].Length != 1)
                throw new ArgumentOutOfRangeException("split");
            Key.Character = split[0][0];
            Key.ControlKeyState = (ControlKeyStates)int.Parse(split[1]);
            Key.VirtualKeyCode = int.Parse(split[2]);
            Key.KeyDown = split[3] == "Y" ? true : false;
            return Key;
        }

        public override void ScrollBufferContents(Rectangle source, Coordinates destination, Rectangle clip, BufferCell fill)
        {
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawScrollBufferContents);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("ScrollBufferContents");
            StringBuilder sb = new StringBuilder();
            sb.Append(source.Top.ToString()+"~");
            sb.Append(source.Bottom.ToString() + "~");
            sb.Append(source.Left.ToString() + "~");
            sb.Append(source.Right.ToString() + "~");
            sb.Append(destination.X.ToString() + "~");
            sb.Append(destination.Y.ToString() + "~");
            sb.Append(clip.Top.ToString() + "~");
            sb.Append(clip.Bottom.ToString() + "~");
            sb.Append(clip.Left.ToString() + "~");
            sb.Append(clip.Right.ToString() + "~");
            sb.Append(fill.Character.ToString() + "~");
            sb.Append(((int)(fill.ForegroundColor)).ToString() + "~");
            sb.Append(((int)(fill.BackgroundColor)).ToString() + "~");
            sb.Append(((int)(fill.BufferCellType)).ToString());

            this.parent.Parent.MyChannel.WriteText(sb.ToString());
            command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("ScrollBufferContents");
        }

        public override void SetBufferContents(Rectangle rectangle, BufferCell fill)
        {
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetBufferContents1);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("SetBufferContents");
            StringBuilder sb = new StringBuilder();
            sb.Append(rectangle.Top.ToString() + "~");
            sb.Append(rectangle.Bottom.ToString() + "~");
            sb.Append(rectangle.Left.ToString() + "~");
            sb.Append(rectangle.Right.ToString() + "~");
            sb.Append(fill.Character.ToString() + "~");
            sb.Append(((int)(fill.ForegroundColor)).ToString() + "~");
            sb.Append(((int)(fill.BackgroundColor)).ToString() + "~");
            sb.Append(((int)(fill.BufferCellType)).ToString());

            this.parent.Parent.MyChannel.WriteText(sb.ToString());
            command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("SetBufferContents");
        }

        public override void SetBufferContents(Coordinates origin, BufferCell[,] contents)
        {
            int Width = contents.GetLength(0);
            int Highth = contents.GetLength(1);
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawSetBufferContents2);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("SetBufferContents2");
            this.parent.Parent.MyChannel.WriteText(origin.X.ToString() + "~" + origin.Y.ToString() + "~" + Width.ToString() + "~" + Highth.ToString());
            command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("SetBufferContents2");
            int BufferCellSize = 14; // sizeof(char) + sizeof(int) * 3;
            byte[] RawData = new byte[Width * Highth * BufferCellSize];
            int k = 0;
            byte[] tmpdata = null;
            for (int i = 0; i < Highth; i++)
            {
                for (int j = 0; j <Width ; j++)
                {
                    tmpdata = BitConverter.GetBytes(contents[i, j].Character);
                    RawData[k] = tmpdata[0];
                    RawData[k + 1] = tmpdata[1];
                    k += 2;
                    tmpdata = BitConverter.GetBytes(((int)contents[i, j].ForegroundColor));
                    RawData[k] = tmpdata[0];
                    RawData[k + 1] = tmpdata[1];
                    RawData[k + 2] = tmpdata[2];
                    RawData[k + 3] = tmpdata[3];
                    k += 4;
                    tmpdata = BitConverter.GetBytes(((int)contents[i, j].BackgroundColor));
                    RawData[k] = tmpdata[0];
                    RawData[k + 1] = tmpdata[1];
                    RawData[k + 2] = tmpdata[2];
                    RawData[k + 3] = tmpdata[3];
                    k += 4;
                    tmpdata = BitConverter.GetBytes(((int)contents[i, j].BufferCellType));
                    RawData[k] = tmpdata[0];
                    RawData[k + 1] = tmpdata[1];
                    RawData[k + 2] = tmpdata[2];
                    RawData[k + 3] = tmpdata[3];
                    k += 4;
                }
            }
            this.parent.Parent.MyChannel.WriteData(RawData, 0, RawData.Length);
            command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("SetBufferContents2");
        }

        public override BufferCell[,] GetBufferContents(Rectangle rectangle)
        {
            int Width, Highth;
            Width = rectangle.Right - rectangle.Left + 1;
            Highth = rectangle.Bottom - rectangle.Top + 1;
            if (Width < 0 || Highth < 0)
                throw new ArgumentOutOfRangeException("rectangle");
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.RawGetBufferContents);
            int command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("GetBufferContents");
            this.parent.Parent.MyChannel.WriteText(rectangle.Top.ToString() + "~" + rectangle.Bottom.ToString() + "~" + rectangle.Left.ToString() + "~" + rectangle.Right.ToString());
            command = this.parent.Parent.MyChannel.ReceiveCommand();
            if (command != SecureChannel.OK)
                throw new InvalidOperationException("GetBufferContents");
            this.parent.Parent.MyChannel.SendCommand(SecureChannel.OK);
            int BufferCellSize = 14; // sizeof(char) + sizeof(int) * 3;
            byte[] RawData = new byte[Width * Highth * BufferCellSize];
            int ReadLength = this.parent.Parent.MyChannel.ReadData(RawData,0,RawData.Length);
            if (ReadLength != RawData.Length)
                throw new InvalidOperationException("GetBufferContents");
            BufferCell[,] results = new BufferCell[Highth,Width];
            int k =0;
            for (int i = 0; i < Highth; i++)
            {
                for (int j = 0; j < Width; j++)
                { 
                    results[i,j].Character = BitConverter.ToChar(RawData,k);
                    k += 2;
                    results[i, j].ForegroundColor = (ConsoleColor)(BitConverter.ToInt32(RawData,k));
                    k += 4;
                    results[i, j].BackgroundColor = (ConsoleColor)(BitConverter.ToInt32(RawData, k));
                    k += 4;
                    results[i, j].BufferCellType = (BufferCellType)(BitConverter.ToInt32(RawData, k));
                    k += 4;
                }
            }
            return results;
        }

    }
}
