using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Net;
using System.Text.RegularExpressions;

namespace PowerShellRemoting
{
    public class Subnet
    {
        private uint mask;
        public uint Mask
        {
            get { return this.mask; }
        }

        private uint address;
        public uint Address
        {
            get { return this.address; }
        }

        public static bool TryParseIP(string IPString, out uint IP)
        {
            IP = 0;
            if (string.IsNullOrEmpty(IPString))
                return false;
            string[] elements = IPString.Split(new Char[] { '.' });
            if (elements.Length != 4)
                return false;
            IP = (uint)Convert.ToInt32(elements[0]) << 24;
            IP += (uint)Convert.ToInt32(elements[1]) << 16;
            IP += (uint)Convert.ToInt32(elements[2]) << 8;
            IP += (uint)Convert.ToInt32(elements[3]);
            return true;
        }

        public Subnet(uint IP, uint IPmask)
        {
            this.address = IP;
            this.mask = IPmask;
        }

        public static bool TryParseSubnet(string SubnetString, out Subnet sub)
        {
            sub = null;
            if (string.IsNullOrEmpty(SubnetString))
                return false;
            string[] data = SubnetString.Split(new char[] { '/' });
            if (data.Length != 2)
                return false;
            uint address = 0;
            uint IPmask = 0;
            if (!Subnet.TryParseIP(data[0], out address))
                return false;
            if (data[1].LastIndexOf('.') == -1)
            {
                if (!uint.TryParse(data[1], out IPmask))
                    return false;
                sub = new Subnet(address, IPmask);
                return true;
            }
            if (!Subnet.TryParseIP(data[1],out IPmask))
                return false;
            sub = new Subnet(address, IPmask);
            return true;
        }

        public override string ToString()
        {
            StringBuilder buf = new StringBuilder();
            buf.Append(((int)this.address & 0xFF000000) >> 24).Append('.');
            buf.Append(((int)this.address & 0x00FF0000) >> 16).Append('.');
            buf.Append(((int)this.address & 0x0000FF00) >> 8).Append('.');
            buf.Append(((int)this.address & 0x000000FF));
            buf.Append('/');
            buf.Append(((int)this.mask & 0xFF000000) >> 24).Append('.');
            buf.Append(((int)this.mask & 0x00FF0000) >> 16).Append('.');
            buf.Append(((int)this.mask & 0x0000FF00) >> 8).Append('.');
            buf.Append(((int)this.mask & 0x000000FF));
            return buf.ToString();
        }

    }
}
