using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using System.Net;

namespace PowerShellRemotingManager
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();            
        }

        private Collection<ServerEntry> ServerList;

        private ServerEntry CurrentEntry = null;

        private Form MessageForm;

        private void LoadServerList()
        {
            this.ServerList = new Collection<ServerEntry>();
            ServerEntry server;
            Microsoft.Win32.RegistryKey software=null, mshforfun=null, servers=null;
            string[] ServerNames, Data;
            string ConfigString;
            try
            {
                software = Microsoft.Win32.Registry.CurrentUser.OpenSubKey("Software", true);
                mshforfun = software.OpenSubKey("MSHForFun");
                if (mshforfun == null)
                {
                    mshforfun = software.CreateSubKey("MSHForFun");
                    servers = mshforfun.CreateSubKey("Servers");
                    servers.SetValue("default", "127.0.0.1;8080;1;;1;0", Microsoft.Win32.RegistryValueKind.String);
                }
                servers = mshforfun.OpenSubKey("Servers");
                ServerNames = servers.GetValueNames();
                foreach (string RegKey in ServerNames)
                {
                    ConfigString = (string)servers.GetValue(RegKey, "");
                    Data = ConfigString.Split(new Char[] { ';' });
                    if (Data.Length != 6)
                        continue;
                    try
                    {
                        server = new ServerEntry(
                            RegKey,
                            new IPEndPoint(IPAddress.Parse(Data[0]), int.Parse(Data[1])),
                            int.Parse(Data[2]),
                            Data[3],
                            Data[4] == "1" ? true : false,
                            Data[5] == "1" ? true : false
                            );
                        this.ServerList.Add(server);
                    }
                    catch (ArgumentNullException)
                    {
                        continue;
                    }
                    catch (ArgumentOutOfRangeException)
                    {
                        continue;
                    }
                }

                if (this.ServerList.Count > 0)
                {
                    foreach (ServerEntry item in this.ServerList)
                    {
                        this.listBox1.Items.Add(item.ServerName);
                    }
                    this.listBox1.SelectedIndex = 0;
                    this.CurrentEntry = this.ServerList[0];
                }

            }
            finally
            {
                if (servers != null)
                    servers.Close();
                if (mshforfun != null)
                    mshforfun.Close();
                if (software != null)
                    software.Close();
            }

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            LoadServerList();
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.CurrentEntry = this.ServerList[this.listBox1.SelectedIndex];
            this.textBox1.Text = this.CurrentEntry.ServerName;
            this.textBox2.Text = this.CurrentEntry.ServerIPEndPoint.Address.ToString();
            this.textBox3.Text = this.CurrentEntry.ServerIPEndPoint.Port.ToString();
            this.label4.ForeColor = Color.Black;
            this.label4.Text = "";
            if (this.CurrentEntry.SecureChannel == 1)
            {
                this.radioButton1.Checked = true;
                this.textBox4.Enabled = false;
            }
            else
            {
               this.radioButton2.Checked = true;
               this.textBox4.Enabled = true;
            }
            this.textBox4.Text = this.CurrentEntry.TargetName;
            if (this.CurrentEntry.CanHandleCancel)
                this.checkBox1.Checked = true;
            else
                this.checkBox1.Checked = false;
            if (this.CurrentEntry.CanThrowException)
                this.checkBox2.Checked = true;
            else
                this.checkBox2.Checked = false;
        }


        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            this.textBox4.Text = "";
            this.textBox4.Enabled = false;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            this.textBox4.Enabled = true;
        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            IPAddress IP;

            if (!IPAddress.TryParse(this.textBox2.Text, out IP))
            {
                this.label4.ForeColor = Color.Red;
                this.label4.Text = "Can't Parse IP Address!";
            }
            else
            {
                this.label4.ForeColor = Color.Black;
                this.label4.Text = "";
            }
        }

        private void textBox3_TextChanged(object sender, EventArgs e)
        {
            int Port = 0;
            if (!int.TryParse(this.textBox3.Text, out Port))
            {
                this.label4.ForeColor = Color.Red;
                this.label4.Text = "Can't Parse Port Number!";
            }
            else if (Port < IPEndPoint.MinPort || Port > IPEndPoint.MaxPort)
            {
                this.label4.ForeColor = Color.Red;
                this.label4.Text = "Invalid Port Number!";
            }
            else
            {
                this.label4.ForeColor = Color.Black;
                this.label4.Text = "";
            }
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(this.textBox1.Text) || this.textBox1.Text.IndexOf(';')>0)
            {
                this.label4.ForeColor = Color.Red;
                this.label4.Text = "Invalid Name!";
                return;
            }
        }

        private void textBox4_TextChanged(object sender, EventArgs e)
        {
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (this.CurrentEntry.ServerName == this.textBox1.Text)
            {
                MessageForm = new Form2("Error", "Entry already exsits. Change entry name and try again.");
                MessageForm.ShowDialog();
                return;
            }
            foreach (ServerEntry item in this.ServerList)
            {
                if (this.textBox1.Text == item.ServerName)
                {
                    MessageForm = new Form2("Error", "Entry already exsits. Change entry name and try again.");
                    MessageForm.ShowDialog();
                    return;
                }
            }
            if (this.radioButton2.Checked && string.IsNullOrEmpty(this.textBox4.Text))
            {
                MessageForm = new Form2("Error", "Target name is required for SSLStream.");
                MessageForm.ShowDialog();
                return;
            }
            Microsoft.Win32.RegistryKey software = null, mshforfun = null, servers = null;
            try
            {
                software = Microsoft.Win32.Registry.CurrentUser.OpenSubKey("Software");
                mshforfun = software.OpenSubKey("MSHForFun");
                servers = mshforfun.OpenSubKey("Servers", true);
                StringBuilder sb = new StringBuilder();
                sb.Append(this.textBox2.Text);
                sb.Append(';');
                sb.Append(this.textBox3.Text);
                sb.Append(';');
                if (this.radioButton1.Checked)
                {
                    sb.Append("1;");
                }
                else
                {
                    sb.Append("2;");
                    sb.Append(this.textBox4.Text);
                }
                sb.Append(';');
                if (this.checkBox1.Checked)
                    sb.Append('1');
                else
                    sb.Append('0');
                sb.Append(';');
                if (this.checkBox2.Checked)
                    sb.Append('1');
                else
                    sb.Append('0');
                servers.SetValue(this.textBox1.Text, sb.ToString(), Microsoft.Win32.RegistryValueKind.String);
                this.listBox1.Items.Clear();
                LoadServerList();
            }
            finally
            {
                if (servers != null)
                    servers.Close();
                if (mshforfun != null)
                    mshforfun.Close();
                if (software != null)
                    software.Close();
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (this.textBox1.Text == "default")
            {
                MessageForm = new Form2("Error", "Can't delete defaut entry!");
                MessageForm.ShowDialog();
                return;
            }
            MessageForm = new Form2("Confirm", "Do you want to delete entry: " + this.textBox1.Text);
            MessageForm.ShowDialog();
            if (MessageForm.DialogResult != DialogResult.OK)
                return;

            Microsoft.Win32.RegistryKey software = null, mshforfun = null, servers = null;
            string[] ServerNames;
            try
            {
                software = Microsoft.Win32.Registry.CurrentUser.OpenSubKey("Software");
                mshforfun = software.OpenSubKey("MSHForFun");
                servers = mshforfun.OpenSubKey("Servers", true);
                ServerNames = servers.GetValueNames();
                foreach (string RegKey in ServerNames)
                {
                    if (RegKey == this.textBox1.Text && RegKey != "default")
                        servers.DeleteValue(RegKey);
                }
                this.listBox1.Items.Clear();
                LoadServerList();
            }
            finally
            {
                if (servers != null)
                    servers.Close();
                if (mshforfun != null)
                    mshforfun.Close();
                if (software != null)
                    software.Close();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            bool IsClientSnapinInstalled;
            string path = PowerShellPath(out IsClientSnapinInstalled);
            if (path == string.Empty)
            {
                MessageForm = new Form2("Error", "Windows PowerShell is not installed!");
                MessageForm.ShowDialog();
                return;
            }
            if (!IsClientSnapinInstalled)
            {
                MessageForm = new Form2("Error", "PowerShell Remoting Client PSSnapin is not installed!");
                MessageForm.ShowDialog();
                return;
            }
            string AppFolder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\MSHForFun";
            string DataFolder = AppFolder + "\\PowerShell Remoting Manager";
            string ScriptFile = DataFolder + "\\LauchScript.ps1";
            string Funcation = 
@"
function Start-RemoteHost
{
    Param ([string] $IPAddress = '127.0.0.1',[int] $Port = 8080, [int] $Channel = 1, [string] $Target = ' ', [bool] $Cancel = $true, [bool] $Exception = $false) 
    $ClientSnapin = Get-PsSnapin | where {$_.Name -eq 'PowerShellRemotingClient'}
    if ($ClientSnapin -eq $null) { Add-PSSnapin 'PowerShellRemotingClient'}
    $RemoteIP = [System.Net.IPAddress]::Parse($IPAddress)
    $RemoteEP = new-object  System.Net.IPEndPoint ($RemoteIP, $Port)
    $RemotingClient = new-object PowerShellRemoting.ClientPSHost($RemoteEP,$host.ui, $Channel, $Target)
    if ($RemotingClient -ne $null)
    {
        if (-not $Cancel) {$RemotingClient.CanHandleCancelKey = $false}
        if ($Exception) {$RemotingClient.CanThrowException = $true}
        $RemotingClient.Start()
    }
    Read-Host 'Press Enter to exit'
}";
            System.IO.StreamWriter sw; 
            if (!System.IO.Directory.Exists(AppFolder))
                System.IO.Directory.CreateDirectory(AppFolder);
            if (!System.IO.Directory.Exists(DataFolder))
                System.IO.Directory.CreateDirectory(DataFolder);
            if (System.IO.File.Exists(ScriptFile))
                System.IO.File.Delete(ScriptFile);
            sw = new System.IO.StreamWriter(ScriptFile);
            StringBuilder sb = new StringBuilder();
            sb.Append("start-remotehost ");
            sb.Append(this.textBox2.Text);
            sb.Append(' ');
            sb.Append(this.textBox3.Text);
            sb.Append(' ');
            if (this.radioButton1.Checked)
            {
                sb.Append("1 ");
            }
            else
            {
                sb.Append("2 ");
                sb.Append(this.textBox4.Text);
            }
            sb.Append(' ');
            if (!this.checkBox1.Checked)
                sb.Append("$false ");
            if (this.checkBox2.Checked)
                sb.Append("$true");

            sw.Write(Funcation);
            sw.WriteLine(sb.ToString());
            sw.Close();
            System.Diagnostics.ProcessStartInfo info = new System.Diagnostics.ProcessStartInfo();
            info.FileName = path;
            info.Arguments = "-command .\\LauchScript.ps1";
            info.WorkingDirectory = DataFolder;
            info.LoadUserProfile = true;
            info.UseShellExecute = false;
            System.Diagnostics.Process.Start(info);
        }

        private string PowerShellPath(out bool ClientPSSnapinInstalled)
        {
            Microsoft.Win32.RegistryKey software = null, microsoft = null, powershell = null, v1 = null, PowerShellEngine = null, PowerShellSnapIns = null, PowerShellRemotingClient = null;
            string path;
            try
            {
                software = Microsoft.Win32.Registry.LocalMachine.OpenSubKey("Software");
                microsoft = software.OpenSubKey("Microsoft");
                powershell = microsoft.OpenSubKey("PowerShell");
                if (powershell == null)
                {
                    ClientPSSnapinInstalled = false;
                    return string.Empty;
                }
                v1 = powershell.OpenSubKey("1");
                PowerShellSnapIns = v1.OpenSubKey("PowerShellSnapIns");
                PowerShellRemotingClient = PowerShellSnapIns.OpenSubKey("PowerShellRemotingClient");
                if (PowerShellRemotingClient != null)
                    ClientPSSnapinInstalled = true;
                else
                    ClientPSSnapinInstalled = false;
                PowerShellEngine = v1.OpenSubKey("PowerShellEngine");
                path = (string)PowerShellEngine.GetValue("ApplicationBase","");
                if (path == "")
                    return string.Empty;
                else
                    return path + "\\powershell.exe";
            }
            finally
            {
                if (PowerShellRemotingClient != null)
                    PowerShellRemotingClient.Close();
                if (PowerShellSnapIns != null)
                    PowerShellSnapIns.Close();
                if (PowerShellEngine != null)
                    PowerShellEngine.Close();
                if (v1 != null)
                    v1.Close();
                if (powershell != null)
                    powershell.Close();
                if (microsoft != null)
                    microsoft.Close();
                if (software != null)
                    software.Close();
            }

        }

    }
}