using System;
using System.Collections.Generic;
using System.Text;
using System.Net;

namespace PowerShellRemotingManager
{
    internal class ServerEntry
    {

        internal ServerEntry(string Name, IPEndPoint ipendpoint, int channel, string target, bool Cancel, bool Exception)
        {
            if (string.IsNullOrEmpty(Name))
                throw new ArgumentNullException("ServerName");
            if (ipendpoint == null)
                throw new ArgumentNullException("IPEndPoint");
            if (channel < 1 || channel > 2)
                throw new ArgumentOutOfRangeException("SecureChannel");
            if (channel == 2 && string.IsNullOrEmpty(target))
                throw new ArgumentNullException("TargetName");
            this.serverName = Name;
            this.serverIPEndPoint = ipendpoint;
            this.secureChannel = channel;
            this.targetName = target;
            this.canHandleCancel = Cancel;
            this.canThrowException = Exception;
        }

        internal ServerEntry()
        {
            this.serverName = "Unamed Server Entry";
            this.serverIPEndPoint = new IPEndPoint(IPAddress.Loopback, 8080);
            this.secureChannel = 1;
            this.targetName = "";
            this.canHandleCancel = true;
            this.canThrowException = false;
        }

        internal ServerEntry(IPEndPoint ipendpoint)
        {
            if (ipendpoint == null)
                throw new ArgumentNullException("IPEndPoint");
            this.serverName = "Unamed Server Entry";
            this.serverIPEndPoint = ipendpoint;
            this.secureChannel = 1;
            this.targetName = "";
            this.canHandleCancel = true;
            this.canThrowException = false;
        }

        internal ServerEntry(string Name, IPEndPoint ipendpoint)
        {
            if (string.IsNullOrEmpty(Name))
                throw new ArgumentNullException("ServerName");
            if (ipendpoint == null)
                throw new ArgumentNullException("IPEndPoint");
            this.serverName = Name;
            this.serverIPEndPoint = ipendpoint;
            this.secureChannel = 1;
            this.targetName = "";
            this.canHandleCancel = true;
            this.canThrowException = false;
        }

        internal ServerEntry(string Name, IPEndPoint ipendpoint, int channel, string target)
        {
            if (string.IsNullOrEmpty(Name))
                throw new ArgumentNullException("ServerName");
            if (ipendpoint == null)
                throw new ArgumentNullException("IPEndPoint");
            if (channel < 1 || channel > 2)
                throw new ArgumentOutOfRangeException("SecureChannel");
            if (channel == 2 && string.IsNullOrEmpty(target))
                throw new ArgumentNullException("TargetName");
            this.serverName = Name;
            this.serverIPEndPoint = ipendpoint;
            this.secureChannel = channel;
            this.targetName = target;
            this.canHandleCancel = true;
            this.canThrowException = false;
        }

        private string serverName;
        internal string ServerName
        {
            get { return this.serverName; }
            set { this.serverName = value; }
        }        
        
        private IPEndPoint serverIPEndPoint;
        internal IPEndPoint ServerIPEndPoint
        {
            get { return this.serverIPEndPoint; }
            set { this.serverIPEndPoint = value; }
        }

        private int secureChannel = 1;
        internal int SecureChannel
        {
            get { return this.secureChannel; }
            set { this.secureChannel = value; }
        }

        private string targetName;
        internal string TargetName
        {
            get { return this.targetName; }
            set { this.targetName = value; }
        }

        private bool canHandleCancel;
        internal bool CanHandleCancel
        {
            get { return this.canHandleCancel; }
            set { this.canHandleCancel = value; }
        }

        private bool canThrowException;
        internal bool CanThrowException
        {
            get { return this.canThrowException; }
            set { this.canThrowException = value; }
        }

    }
}
